/****************************************************************************
 * $Author: gecko $
 * $Revision: 1.1 $
 * $Log: parsecmd.c,v $
 * Revision 1.1  1998/04/22 16:45:00  gecko
 * Initial revision
 *
 *
This file is free software and is part of Gecko's personal library; 
you can redistribute it and/or modify it under the terms of the GNU
General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later verson.
             
This file is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
                             
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

for information on how to use the function here, write the author at
xbio@benham.net
                                                                                                          
Name      Vers  Date        Author      Notes
--------  ----  ----------  ----------- --------------------------------------
parsecmd  v1.1  04/22/1997  The Gecko   Initial version
****************************************************************************/
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	"parsecmd.h"

/****************************************************************************
**  FillTemplate - Fills the destination variable based on data given
**
**  Parameters:	Name		Description
**		__________________________________________________________
**		szData		Data to be added
**		pTemplates	Template to add to
**
**  Returns:	1 on success, 0 on not found, -1 on error
****************************************************************************/
int	FillTemplate( char *szData, CMD_TEMPLATE *pTemplate )
{			
	switch( pTemplate->fDestType )
	{
		case DEST_STRING:
			if ( pTemplate->fParameter )
			{
				if ( !szData )
					return -1;
				strcpy( pTemplate->pDest, szData );
				return 1;
			}	/*if*/
			else
				return -1;
		case DEST_INT:
			if ( pTemplate->fParameter )
			{
				if ( !szData )
					return -1;
				*(int*)(pTemplate->pDest) = atoi( szData );
				return 1;
			}	/*if*/
			else
				return -1;
		case DEST_FLOAT:
			if ( pTemplate->fParameter )
			{
				if ( !szData )
					return -1;
				*(float*)(pTemplate->pDest) = atof( szData );
				return 1;
			}	/*if*/
			else
				return -1;
		case DEST_FLAG:
			if ( pTemplate->fParameter )
			{
				if ( !szData )
					return -1;
				if ( *szData == 'n' || *szData == 'N' || *szData == 'f' || *szData == 'F' || *szData == '0' )
					*(char*)(pTemplate->pDest) = 0;
				else
					*(char*)(pTemplate->pDest) = !0;
			}	/*if*/
			else
			{
				if ( szData && szData[0] == '-' )
					*(char*)(pTemplate->pDest) = 0;
				else
					*(char*)(pTemplate->pDest) =!0;
        	}	/*else*/
			return 1;
	}	/*switch*/
	return -1;
}	/*function*/

/****************************************************************************
**  FindName - Searches template list for given name and take proper action
**
**  Parameters:	Name		Description
**		__________________________________________________________
**		szBuffer		String to match
**		pTemplates	Template Rules to match
**
**  Returns:	1 on success, 0 on not found, -1 on error
****************************************************************************/
int	FindName( char *szString, CMD_TEMPLATE *pTemplates )
{
	char	szBuffer[MAX_NAME_STRING];
	char	*szName = szBuffer;
	char	*szData = NULL;
	
	/*parse data string for name and data elements*/
	if ( strlen( szString ) >= sizeof( szBuffer ) )	/*exceeds internal buffer*/
		return 0;
	strcpy( szBuffer, szString );
	szData = strchr( szBuffer, '=' );
	*szData = 0;
	szData++;
	
	/*look for name in list of templates*/
	while ( pTemplates->pDest )
	{
		if ( pTemplates->szName )
		{
			if ( strcmp( pTemplates->szName, szName ) ==0 )
			{
				return FillTemplate( szData, pTemplates );
			}	/*if*/
		}	/*if*/
		pTemplates++;
	}	/*while*/
	return	0;
}	/*function*/

/****************************************************************************
**  GetSwitch - Searches template list for given name and take proper action
**
**  Parameters:	Name	Description
**		__________________________________________________________
**		szBuffer		Current Place in Commandline list
**		pTemplates	Template Rules to match
**
**  Returns:	1 on success, 0 on not found, -1 on error
****************************************************************************/
int	GetSwitch( char **szString, CMD_TEMPLATE *pTemplatesIn, FILE_LIST **pFileNames )
{
	int		i;
	char		*szBuffer, szData[MAX_NAME_STRING];
	FILE_LIST	*pBuffer, *pWork;
	CMD_TEMPLATE*pTemplates;

	/*check for switch or for non switch*/
	if ( (*szString)[0] == '-' )
	{
		if ( (*szString)[1] == '-' )
		{
			/*long name parameter*/
		}	/*if*/
		else
		{
			for ( i=1; i<strlen( *szString ); i++ )
			{
				pTemplates = pTemplatesIn;
				while ( pTemplates->pDest )
				{
					if ( pTemplates->szSwitch )
					{
						if ( pTemplates->szSwitch[0] == (*szString)[i] )
						{
							if ( pTemplates->fParameter )
							{
								if ( (*szString)[i+1] == 0 )
								{
									if ( szString[1] == NULL )
										return -1;
									if ( strlen( szString[1] ) >= MAX_NAME_STRING )
										return -1;

									strcpy( szData, szString[1] );
									if ( FillTemplate( szData, pTemplates ) < 0 )
										return -1;
									else
										return 1;
								}	/*if*/
								else
									strcpy( szData, (*szString)+i+1 );
							}	/*if*/
							else
							{
								if ( (*szString)[i+1] == '-' )
								{
									szData[0]='-';
									i++;
								}	/*if*/
								else
									szData[0]=0;
							}	/*else*/
							if ( FillTemplate( szData, pTemplates ) < 0 )
								return -1;
							i = strlen( *szString );
							break;
						}	/*if*/
					}	/*if*/
					pTemplates++;
				}	/*while*/
			}	/*for*/
		}	/*else*/
	}	/*if*/
	else
	{
		/*do we allow unswitched parameters*/
		if ( pFileNames == NULL )
			return -1;

		/*allocate new node for list*/
		pBuffer = (FILE_LIST*)malloc( sizeof( FILE_LIST ) );
		if ( pBuffer == NULL )
		{
			fprintf( stderr, "Not enough memory to process filename list\n" );
			return -1;
		}	/*if*/
		
		/*allocate space for string*/
		szBuffer = (char *)malloc( strlen( *szString )+1 );
		if ( szBuffer == NULL )
		{			
			fprintf( stderr, "Not enough memory to process filename list\n" );
			free( pBuffer );
			return -1;
		}	/*if*/
		
		/*fill in node*/
		strcpy( szBuffer, *szString );
		pBuffer->szFilename = szBuffer;
		pBuffer->pNext = NULL;
		
		/*add node to list*/
		if ( *pFileNames == NULL )
		{
			*pFileNames = pBuffer;
			pBuffer->pPrev  = NULL;
		}	/*if*/
		else
		{
			pWork = *pFileNames;
			while( pWork->pNext )
				pWork = pWork->pNext;
			pWork->pNext = pBuffer;
			pBuffer->pPrev = pWork;
		}	/*else*/		
	}	/*else*/
	
	return	0;
}	/*function*/

/****************************************************************************
**  Parse Cmd - Parses the Command Line. 
**
**  Parameters:	Name	Description
**		__________________________________________________________
**		nArgs	Number of Arguments (from main parameters)
**		lpszArgs	Commandline Arguments (from main parameters)
**		lpszEnv	Enviroment Arguments (from main paramters)
**		lpszFiles	RC Files to process
**
**  Returns:	1 on success, -1 on parameter Error
****************************************************************************/
int	ParseCmd( int nArgs, char **lpszArgs, char **lpszEnv, char **lpszFiles, CMD_TEMPLATE *pTemplate, FILE_LIST **pFileList )
{
	FILE	*pFile;
	int	i;
	char	szBuffer[MAX_NAME_STRING];

	/*process environment variables*/
	if ( lpszEnv )
		while( *lpszEnv )
		{
			if ( FindName( *lpszEnv, pTemplate  ) < 0 )
				return -1;
			lpszEnv++;
		}	/*while*/

	/*process rc files*/
	if ( lpszFiles )
		while( *lpszFiles )
		{
			pFile = fopen( *lpszFiles, "r" );
			if ( pFile )
			{
				while ( !feof( pFile ) )
				{
					fgets( szBuffer, sizeof( szBuffer ), pFile );
					szBuffer[strlen( szBuffer )-1] = 0;
					if ( FindName( szBuffer, pTemplate  ) < 0 )
						return -1;
				}	/*while*/
				fclose( pFile );
			}	/*if*/
			lpszFiles++;
		}	/*while*/

	/*process command line*/
	for ( i=0; i<nArgs; i++ )
	{
		int	nResult;

		nResult = GetSwitch( lpszArgs+i, pTemplate, pFileList );
		if ( nResult < 0 )
			return -1;
		if ( nResult > 0 )
			i++;
	}	/*for*/
	return 1;
}	/*function*/
