/*************************************************************************
XBio - an X program to show Biorhythm graphs for an individual.
Copyright (C) 1995,1996 Alex Butcher
Email: Alex.Butcher@bris.ac.uk
Smail: Computing Service, University of Bristol, Tydall Ave., Clifton,
       Bristol, England.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
             
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
                             
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
                                                                                                          
Name  Vers  Date        Author      Notes
----  ----  ----------  ----------- --------------------------------------
Xbio  v0.1  12/12/1995  A.J.Butcher Initial version

      v1.0  14/12/1995              Debugged version, with 'Age in days'
                                    display and quit button (XBio v1.0)
                
      v1.1  29/09/1996              Re-Worked for XForms 0.81 API
                                    (XBio v1.1)
      v2.0  30/03/1998	Gecko       Changed today to go by month - added
                                    command-line options - added print
      v2.1  24/07/2003              Recompiled for XForms 1.x
*************************************************************************/

#ifndef FL_FORMS_H
#include <forms.h>
#endif

#ifndef FD_xbio_h_
#include "xbio_gui.h"
#endif

#include "parsecmd.h"

#ifndef _XLIB_H_
#include <X11/Xlib.h>
#endif

#ifndef _UNISTD_H
#include <unistd.h>
#endif

#ifndef _STDIO_H
#include <stdio.h>
#endif

#ifndef _STDLIB_H
#include <stdlib.h>
#endif

#ifndef _STRING_H
#include <string.h>
#endif

#ifndef _MATH_H
#include <math.h>
#endif

#ifndef _TIME_H
#include <time.h>
#endif

#ifndef M_PI
#define M_PI (3.14159265358979323846)
#endif

#define ELAYER (FL_MAX_XYPLOTOVERLAY - 3)
#define PLAYER (FL_MAX_XYPLOTOVERLAY - 2)
#define ILAYER (FL_MAX_XYPLOTOVERLAY - 1)
#define ALAYER (FL_MAX_XYPLOTOVERLAY - 0)

/* Global variables */
/* XForms Form Descriptor pointer */
FD_xbio *fd_xbio;

/* Month lengths */
const int days_in[] = { 31,28,31,30,31,30,31,31,30,31,30,31 };

const char month[13][4] = { "Err", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
/* Birthdate variables */
int bdd=0,bmm=0,byyyy=0;

/* Todays date variables */
int tdd=0,tmm=0,tyyyy=0;

/* Buttons' status */
int emot_stat=TRUE,phys_stat=TRUE,int_stat=TRUE,av_stat=TRUE;

/* Graph status - drawn last time round? */
int edraw=TRUE,pdraw=TRUE,idraw=TRUE,adraw=TRUE;

/* Graph data */
float delta[33],e[33],i[33],p[33],av[33],dummy[33];

/* Print Program */
char lpr[MAX_NAME_STRING] = "lpr";

/* Function prototypes */
void parse_commandline( int argc, char *argv[] );
int  verify_date( const char *szDate, int *month, int *day, int *year );
void biograph (void);
int is_leap_year (int year);
int days_in_month (int month, int year);
int days_between_years (int styear, int enyear);
int days_between_months (int stmonth, int enmonth, int year);
int days_old (void);

void	help( void )
{
	printf( "XBio [-b date] [-t date] [-p command]\n" );
	printf( "    -b date          - set birthdate to \"date\"  (DD/MM/YY or DD/MM/YYYY format only!)\n" );
	printf( "    -t date          - set month to plot for to \"date\"  (MM/YY or MM/YYYY format only!)\n" );
	printf( "    -l command       - print command to use (default = \"lpr\")\n" );
	exit( 0 );
}	/*function*/

int main( int argc, char *argv[] )
{
  int i;
  char	szBuf[21];

  printf("XBio version 2.1, Copyright (C) 1995-1996 Alex Butcher.\n");
  printf("                  Copyright (C) 1998-2003 Darren Benham.\n");
  printf("XBio comes with ABSOLUTELY NO WARRANTY; for details read COPYING.\n");
  printf("This is free software, and you are welcome to redistribute it\n");
  printf("under certain conditions; read COPYING for details.\n\n");

  /* initialize the arrays for dummy data sets */
  for(i=0; i<=31; i++)
  {
    delta[i]=(float)(i);
    dummy[i]=(float)0.0;
  }
  /* XForms stuff */
  fl_initialize(&argc, argv, "XBio v2.1", 0, 0);
  fd_xbio=create_form_xbio();
  
  verify_date( "now", &tmm, NULL, &tyyyy );
  parse_commandline( argc, argv );

  /* Put a zero in the 'age in days' field... */
  sprintf( szBuf, "%i", days_old() );
  fl_set_input(fd_xbio->dayage, szBuf);
  /* ... and prevent the user from entering stuff in it! */
  fl_deactivate_object(fd_xbio->dayage);

  /* initialization of widgets */
  sprintf( szBuf, "%.2i/%.2i/%.4i", bdd, bmm, byyyy );
  fl_set_input(fd_xbio->birthday, szBuf );
  sprintf( szBuf, "%.2i/%.4i", tmm, tyyyy );
  fl_set_input(fd_xbio->today, szBuf);
  fl_set_button(fd_xbio->emot_check, TRUE);
  fl_set_button(fd_xbio->phys_check, TRUE);
  fl_set_button(fd_xbio->int_check, TRUE);
  fl_set_button(fd_xbio->av_check, TRUE);

  /* dummy graphs */
  fl_set_xyplot_data(fd_xbio->biochart, delta, dummy, 32, "", "", "");
  fl_add_xyplot_overlay(fd_xbio->biochart, 9, delta, dummy, 32, FL_GREEN);
  fl_add_xyplot_overlay(fd_xbio->biochart, 10, delta, dummy, 32, FL_RED);
  fl_add_xyplot_overlay(fd_xbio->biochart, 11, delta, dummy, 32, FL_BLUE);
  fl_add_xyplot_overlay(fd_xbio->biochart, 12, delta, dummy, 32, FL_WHITE);

  /* window manager stuff */
  fl_show_form(fd_xbio->xbio, FL_PLACE_FREE, FL_FULLBORDER, "XBio v2.0");

  /* make sure that everything gets drawn properly! */
  XFlush( fl_get_display() );

  /* leave XForms to handle user input */
  fl_do_forms();

  /* er... just in case! */
  return(0);
}

void parse_commandline( int argc, char *argv[] )
{
	int		i;
	char	szBirth[MAX_NAME_STRING]="";
	char	szToday[MAX_NAME_STRING]="";
	char	fHelp;
	CMD_TEMPLATE	pCmdTemplate[] = {
	{ 1, 1, "b", "BIRTHDATE", NULL },
	{ 1, 1, "t", "TODAY", NULL },
	{ 1, 1, "l", NULL, lpr },
	{ 0, 0, "h", NULL, NULL },
	{ 0, 0, NULL, NULL, NULL }};

	/*parse commandline*/
	pCmdTemplate[0].pDest = szBirth;
	pCmdTemplate[1].pDest = szToday;
	pCmdTemplate[3].pDest = &fHelp;
	i = ParseCmd( argc-1, argv+1, NULL, NULL, pCmdTemplate, NULL );

	/*check for error or help message*/
	if ( i < 0 || fHelp )
	{
		if ( i < 0 )
			fprintf( stderr, "Bad Commandline Options\n" );
		help();
		return;
	}	/*if*/

	/*process commandline dates*/
	if ( strlen( szBirth ) )
		verify_date( szBirth, &bmm, &bdd, &byyyy );
	if ( strlen( szToday ) )
		verify_date( szToday, &tmm, NULL, &tyyyy );
}	/*function*/

void biograph(void)
{
  int days;

  /* save some calculation time! */
  double twopi28 = ((double)2.0 * M_PI)/(double)28.0;
  double twopi33 = ((double)2.0 * M_PI)/(double)33.0;
  double twopi23 = ((double)2.0 * M_PI)/(double)23.0;

  /* this is a delta counter from todays date onwards for 31 days */
  int day;

  /* used for the 'age in days' widget */
  char agestr[11];

  /* how old is the user in days? */
  days=days_old()-1;

  /* nonsense! */
  if (days<0)
  {
    return;
  }

  /* enter the age in days in the appropriate widget. */
  /* they only take char * BTW */
  sprintf(agestr,"%d",days+1);
  fl_set_input(fd_xbio->dayage,agestr); 

  /* sine wave of period 28 days for emotional cycle */
  for (day=(days%28); day<=(days%28)+32; day++)
  {
    e[day-(days%28)]=(float)sin((double)day*twopi28);
  }

  /* period 33 days for intellectual cycle */
  for (day=(days%33); day<=(days%33)+32; day++)
  {
    i[day-(days%33)]=(float)sin((double)day*twopi33);
  }

  /* period 23 days for physical cycle */
  for (day=(days%23); day<=(days%23)+32; day++)
  {
    p[day-(days%23)]=(float)sin((double)day*twopi23);
  }

  /* average */
  for (day=0; day<=32; day++)
  {
    av[day]=(((float)e[day]+(float)i[day]+(float)p[day])/(float)3.0);
  }
    

  /* prevent as much flicker as possible */
  fl_freeze_form(fd_xbio->xbio);

  /* if we drew these graphs last time, delete them */
  if (adraw) { fl_delete_xyplot_overlay(fd_xbio->biochart, ALAYER); }
  if (idraw) { fl_delete_xyplot_overlay(fd_xbio->biochart, ILAYER); }
  if (pdraw) { fl_delete_xyplot_overlay(fd_xbio->biochart, PLAYER); }
  if (edraw) { fl_delete_xyplot_overlay(fd_xbio->biochart, ELAYER ); }

  /* plot a dummy graph, seeing as we use overlays for the real data */
  fl_set_xyplot_data(fd_xbio->biochart, delta, dummy, 32, "", "", "");

  /* draw 'em if the user wants and make a note whether we did or not */
  if (emot_stat)
  {    
    fl_add_xyplot_overlay(fd_xbio->biochart, ELAYER, delta, e, 32, FL_GREEN );
    edraw=TRUE;
  }
  else
  {
    edraw=FALSE;
  }

  if (phys_stat)
  {
    fl_add_xyplot_overlay(fd_xbio->biochart, PLAYER, delta, p, 32, FL_RED);
    pdraw=TRUE;
  }
  else
  {
    pdraw=FALSE;
  }

  if (int_stat)
  {
    fl_add_xyplot_overlay(fd_xbio->biochart, ILAYER, delta, i, 32, FL_BLUE);
    idraw=TRUE;
  }
  else
  {
    idraw=FALSE;
  }

  if (av_stat)
  {
    fl_add_xyplot_overlay(fd_xbio->biochart, ALAYER, delta, av, 32, FL_WHITE);
    adraw=TRUE;
  }
  else
  {
    adraw=FALSE;
  }

  /* show the XYPLOT again - this line caused a LOT of pain! */
  fl_show_object(fd_xbio->biochart);

  /* update the window all in one go */
  fl_unfreeze_form(fd_xbio->xbio);
}

int is_leap_year(int year)
{
  /* leap centuries */
  if ((year % 400) == 0)
  {
    return(TRUE);
  }

  /* leap years that aren't centuries (1800 was NOT a leap year!) */
  if ( (year % 4 == 0) && (year % 100 != 0) )
  {
    return(TRUE);
  }
  else
  {
    return(FALSE);
  }
}

int days_in_month(int month, int year)
{
  /* you need comments for this routine?! :) */

  if (month != 2)
  {
    return(days_in[month-1]);
  }
  else
  {
    if (is_leap_year(year))
    {
      return(29);
    }
    else
    {
      return (days_in[month-1]);
    }
  }
}

int days_between_years(int styear, int enyear)
{
  int days=0;
  int year;
  
  if(styear > enyear)
  {
    return(0);
  }
  else
  {
    for(year=styear; year<=enyear; year++)
    {
      if (is_leap_year(year))
      {
        days=days+366;
      }
      else
      {
        days=days+365;
      }
    }
  }
  return(days);
}

int days_between_months(int stmonth, int enmonth, int year)
{
  int days=0;
  int month;

  if (stmonth > enmonth)
  {
    return(0);
  }
  else
  {
    for (month=stmonth; month<=enmonth; month++)
    {
      days=days+days_in_month(month, year);
    }
  }
  return(days);
}

int days_old(void)
{
  int days=0;

  /* handles most cases */
  if (tyyyy > byyyy)
  {
    days=days+days_between_years(byyyy+1, tyyyy-1);
    days=days+days_between_months(bmm+1, 12, byyyy);
    days=days+days_between_months(1, tmm-1, tyyyy);
    days=days+tdd;
    days=days+(days_in_month(bmm,byyyy) - bdd);
  }
  else
  {
    /* handles cases where tyyyy = byyyy */
    if (tmm > bmm)
    {
      days=days+days_between_months(bmm+1, tmm-1, byyyy);
      days=days+tdd;
      days=days+(days_in_month(bmm,byyyy) - bdd);
    }
    else
    {
      /* handles cases where both are in the same month */
      days=tdd-bdd;
    }
  }
  return(days);
}
      
int  verify_date( const char *szDate, int *month, int *day, int *year )
{
  	int convs;
	char	szBuff[11];
  
  	/* Bloat!? Nah, gen-u-ine easter eggs, mate... :) */
  	if (strcmp( szDate, "adam" ) == 0)
  	{
    	szDate = "05/12/1973";
  	}
    
	if (strcmp( szDate , "rob" ) == 0)
  	{
    	szDate = "18/12/1972";
  	}
    
  	if (strcmp( szDate, "jon" ) == 0)
  	{
    	szDate = "23/03/1974";
  	}
  
  	if (strcmp( szDate, "jin" ) == 0)
  	{
    	szDate = "10/06/1973";
  	}
  
  	if (strcmp( szDate, "alex" ) == 0)
  	{
    	szDate = "18/03/1974";
  	}

  	if (strcmp( szDate, "gecko" ) == 0)
  	{
    	szDate = "20/10/1968";
  	}

  	if (strcmp( szDate, "cynthia" ) == 0)
  	{
    	szDate = "07/07/1970";
  	}

	if (strcmp( szDate, "now" ) == 0)
	{
		time_t	tTime = time( NULL );
		struct tm	*lptm;

		lptm = localtime( &tTime );
		sprintf( szBuff, "%.2i/%.4i", lptm->tm_mon+1, lptm->tm_year+1900 );
		szDate = szBuff;
	}	/*if*/

  	/* try and do some fairly intelligent parsing */
	if ( day )
	{
  		convs = (sscanf( szDate," %2d %*[/] %2d %*[/] %4d ", day, month, year ) != 3);
	}	/*if*/
	else
	{
  		convs = (sscanf( szDate," %2d %*[/] %4d ", month, year ) != 2);
		tdd=1;
	}	/*else*/

	if ( *year < 100 )
		*year += 1900;

	if ( convs )
	{
    	/* what DID you enter?! */
		if ( day )
	    	fl_show_alert( "Conversion error!","Birth date not in (dd/mm/yyyy) format","",FALSE );
		else
	    	fl_show_alert( "Conversion error!","Today's date not in (mm/yyyy) format","",FALSE );
		
    	/* now be serious! prompt the user to try again */ 
    	return 0;
  	}	/*if*/

  if ((*month < 1) || (*month > 12))
  {
    fl_show_alert("Conversion error!","Date MONTH out of range (1-12)","",FALSE);
    return 0;
  }

  /* 1752 was when the current calendar began - I can't remember the */
  /* date, so 1753 is the first year I allow! */
  if ( *year <= 1752 )
  {
    fl_show_alert("Conversion error!","Date YEAR <= 1752","",FALSE);
    return 0;
  }
	
	if ( day )
	{
  		if (( *day < 1) || (*day > days_in_month(*month,*year)))
  		{
    		fl_show_alert("Conversion error!","Date DAY out of range for given MONTH","",FALSE);
		    return 0; 
  		}
	}	/*if*/

  /* attempt to redraw the graph when anything changes in the birth date field */
  biograph();
  return 1;
}

void birthday_cb(FL_OBJECT *obj, long arg)
{
	if ( verify_date( fl_get_input(obj), &bmm, &bdd, &byyyy ) == 0 )
	    fl_set_object_focus(fd_xbio->xbio,obj);
}	/*function*/

void today_cb(FL_OBJECT *obj, long arg)
{
	if ( verify_date( fl_get_input(obj), &tmm, NULL, &tyyyy ) == 0 )
  		fl_set_object_focus(fd_xbio->xbio,obj);
	tdd = 1;
}	/*function*/

void emot_cb(FL_OBJECT *obj, long arg)
{
  emot_stat=fl_get_button(obj);
  biograph();
}

void phys_cb(FL_OBJECT *obj, long arg)
{
  phys_stat=fl_get_button(obj);
  biograph();
}

void int_cb(FL_OBJECT *obj, long arg)
{
  int_stat=fl_get_button(obj);
  biograph();
}

void av_cb(FL_OBJECT *obj, long arg)
{
  av_stat=fl_get_button(obj);
  biograph();
}

int	round( float fNumber )
{
	int	x, y;

	if ( fNumber < 0 )
	{
		fNumber *= -1;
		y = -1;
	}	/*if*/
	else
		y = 1;

	x = fNumber * 100;
	if ( (x % 10) > 4 )
		x += 10;

	return (int)(x/10)*y;
		
}	/*function*/

void print_cb( FL_OBJECT *obj, long arg )
{
	int	x, np, ni, ne;
	char	szBuf[81];
	FILE	*pFile;
	
	pFile = popen( lpr, "w" );
	if ( pFile == 0 )
	{
		fprintf( stderr, "Unable to open printer\n" );
		return;
	}	/*if*/
	fprintf( pFile, "Biorythm for %s %u\n", month[tmm], tyyyy );
	fprintf( pFile, "day | Negative 0 Positive | Raw Data         Critical Days\n" );
	for ( x=1; x<32; x++ )
	{
		sprintf( szBuf, "%2i: |          |          |", x );
		np = round( p[x] );
		ni = round( i[x] );
		ne = round( e[x] );

		szBuf[10+np+5] = 'P';
		szBuf[10+ni+5] = 'I';
		szBuf[10+ne+5] = 'E';

		fprintf( pFile, szBuf );
		fprintf( pFile, " P%+3.0f  I%+3.0f  E%+3.0f", p[x]*10, i[x]*10, e[x]*10 );

		if ( x > 0 )
		{
			if ( (p[x] > 0 && p[x+1] < 0) || (p[x] < 0 && p[x+1] > 0) || p[x] == 0 )
				fprintf( pFile, " Physical");
			if ( (i[x] > 0 && i[x+1] < 0) || (i[x] < 0 && i[x+1] > 0) || i[x] == 0 )
				fprintf( pFile, " Intellectual" );
			if ( (e[x] > 0 && e[x+1] < 0) || (e[x] < 0 && e[x+1] > 0) || e[x] == 0 )
				fprintf( pFile, " Emotional" );
		}	/*if*/
		fprintf( pFile, "\n" );
	}	/*for*/
	fprintf( pFile, "P=Physical I=Intellectual E=Emotional\n" );
	fclose( pFile );
}

void prev_cb( FL_OBJECT *obj, long arg )
{
	char	szBuf[11];
	if ( tmm == 0 || tyyyy == 0 )
		verify_date( "now", &tmm, NULL, &tyyyy );

	if ( tmm > 1 )
		tmm--;
	else
	{
		tmm = 12;
		tyyyy--;
	}	/*else*/

  	sprintf( szBuf, "%.2i/%.4i", tmm, tyyyy );
  	fl_set_input(fd_xbio->today, szBuf);
/*    XFlush( fl_get_display() );*/
	biograph();
}

void next_cb( FL_OBJECT *obj, long arg )
{
	char	szBuf[11];
	if ( tmm==0 || tyyyy==0 )
		verify_date( "now", &tmm, NULL, &tyyyy );

	if ( tmm < 12 )
		tmm++;
	else
	{
		tmm = 1;
		tyyyy++;
	}	/*else*/

	sprintf( szBuf, "%.2i/%.4i", tmm, tyyyy );
	fl_set_input( fd_xbio->today, szBuf );
    /*XFlush( fl_get_display() );*/
	biograph();
}

void info_cb(FL_OBJECT *obj, long arg)
{
  /* fairly graceful exit */
  fl_show_alert( "Info XBio 2.1", "Written 1995-1996  by\nAlex Butcher using XForms", "Modified (1998-2003) by Gecko (xbio@benham.net)", FALSE ); 
}

void quit_cb(FL_OBJECT *obj, long arg)
{
  /* fairly graceful exit */
  fl_hide_form(fd_xbio->xbio);
  exit(0);
}
